@@ -22,6 +22,10 @@ import read_vme_offline.general as general
 
 from shutil import copyfile
 
+import sys
+
+import ROOT
+
 
 #>>> import pandas as pd
 #>>> import matplotlib.pyplot as plt
@@ -227,7 +231,7 @@ def fastread(filename, x=0, y=1, batch = 0, read_last=0, df = None, plot = True)
 
 
 #=============================== MAIN1 EVA ==================
-def eva_cut_time(filename,  od=0, do=9999999 , chan=1):
+def eva_cut_time(filename,  od=0, do=9999999 , chan=1,  tree = False):
     """
     EVA: use: read_vme_offline cut1 filename_with_asc  60 120
     """
@@ -235,8 +239,8 @@ def eva_cut_time(filename,  od=0, do=9999999 , chan=1):
     # do = 999999
 
 
-    # start = global_filename_decomp(filename)   # GLOBAL CALL
-    start = general.filename_decomp(filename)   # GLOBAL CALL
+    #*************
+    start = general.filename_decomp(filename)   #  - get start
 
 
     print(f"D... real start",start)
@@ -251,6 +255,7 @@ def eva_cut_time(filename,  od=0, do=9999999 , chan=1):
     print(f"D... CUT  stop  {stopcut}  (demanded or max)")
 
 
+    #*************
     nlines   = general.get_number_of_lines(filename)
     ncolumns = general.get_number_of_columns(filename)
     print(f"i... reading the table of {nlines} lines ... ({nlines/1e+6:.1f} milion lines)")
@@ -258,6 +263,7 @@ def eva_cut_time(filename,  od=0, do=9999999 , chan=1):
 
 
     # df = pd.read_table( filename, names=['time','E','x','ch','y'], sep = "\s+", comment="#")
+    #*************
     df = general.pd_read_table(filename)
     print(df.dtypes)
     print(df)
@@ -277,12 +283,16 @@ def eva_cut_time(filename,  od=0, do=9999999 , chan=1):
 
     # print(f"D...  numpy array for ch {chan}  ________________________CUT___________")
     df1.reset_index(inplace=True, drop=True)
+
+    df1.fillna(0,inplace=True)
+
     print(df1)
     if len(df1)==0:
         print("D... no data for channel {chan}")
         sys.exit(0)
 
     # dfzero = df1[ (df1.E==0) ]
+    #*************
     len_dfzero = general.pd_detect_zeroes(df1) # df1[ (df1.E==0) ]
 
     print()
@@ -335,29 +345,24 @@ def eva_cut_time(filename,  od=0, do=9999999 , chan=1):
 
 
 
-    bname = os.path.splitext(filename)[0]
+    bname = os.path.splitext(filename)[0]  #basename
+    # hname = bname.split("_")[-1]  # last thing should be a comment
+    #*************
+    hname = general.generate_hname(filename, chan)
+
     outfile = bname+".txt"
-    hname = bname.split("_")[-1]
-    his = general.column_to_histo(df2['E'], savename = outfile, hname = hname)
+    #*************
+    his = general.column_to_histo(df2['E'], savename = outfile, hname = hname, writeondisk = True)
     outfile2 = bname+".asc1"
-    # his = general.column_to_histo(df2["E"], savename = outfile, hname = hname)
+    print(f"D... {outfile2} is save as a duplicate")
     copyfile(outfile, outfile2)
 
-    #narr = df2.E.to_numpy()
-    #    print("D... to_numpy", len(narr), narr.dtype)
-    #print(narr)
 
-    #print(f"D...  creating histogram for channel {chan} from numpy")
-
-    #his,bins = np.histogram(narr,bins=1024*32,range=(0.0,1024*32) )
-    #print()
-    #print(his)
-    #print(bins)
-    #print()
-
-    #print(f"D... creating text spectrum = {outfile}")
-    #np.savetxt(outfile, his, fmt="%d")
 
+    if tree:
+        #*************
+        general.save_to_tree(df2, filename) #
+    return
 
 
 if __name__=="__main__":
